@@ -27,6 +27,8 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 #include <libsoup/soup.h>
+#include <libedataserver/libedataserver.h>
+
 #include "e-soap-message.h"
 #include "ews-errors.h"
 #include "e-ews-folder.h"
@@ -115,6 +117,19 @@ typedef enum {
 	E_EWS_BODY_TYPE_TEXT
 } EEwsBodyType;
 
+typedef enum {
+	E_EWS_SIZE_REQUESTED_UNKNOWN = 0,
+	E_EWS_SIZE_REQUESTED_48X48 = 48,
+	E_EWS_SIZE_REQUESTED_64X64 = 64,
+	E_EWS_SIZE_REQUESTED_96X96 = 96,
+	E_EWS_SIZE_REQUESTED_120X120 = 120,
+	E_EWS_SIZE_REQUESTED_240X240 = 240,
+	E_EWS_SIZE_REQUESTED_360X360 = 360,
+	E_EWS_SIZE_REQUESTED_432X432 = 432,
+	E_EWS_SIZE_REQUESTED_504X504 = 504,
+	E_EWS_SIZE_REQUESTED_648X648 = 648
+} EEwsSizeRequested;
+
 typedef struct {
 	gchar *id;
 	gchar *dn;
@@ -391,9 +406,11 @@ void		ews_oal_free			(EwsOAL *oal);
 void		ews_oal_details_free		(EwsOALDetails *details);
 
 GType		e_ews_connection_get_type	(void);
-EEwsConnection *e_ews_connection_new		(const gchar *uri,
+EEwsConnection *e_ews_connection_new		(ESource *source,
+						 const gchar *uri,
 						 CamelEwsSettings *settings);
-EEwsConnection *e_ews_connection_new_full	(const gchar *uri,
+EEwsConnection *e_ews_connection_new_full	(ESource *source,
+						 const gchar *uri,
 						 CamelEwsSettings *settings,
 						 gboolean allow_connection_reuse);
 void		e_ews_connection_update_credentials
@@ -405,7 +422,12 @@ ESourceAuthenticationResult
 						 const ENamedParameters *credentials,
 						 GCancellable *cancellable,
 						 GError **error);
+ESource *	e_ews_connection_get_source	(EEwsConnection *cnc);
 const gchar *	e_ews_connection_get_uri	(EEwsConnection *cnc);
+ESoupAuthBearer *
+		e_ews_connection_ref_bearer_auth(EEwsConnection *cnc);
+void		e_ews_connection_set_bearer_auth(EEwsConnection *cnc,
+						 ESoupAuthBearer *bearer_auth);
 const gchar *	e_ews_connection_get_password	(EEwsConnection *cnc);
 gchar *		e_ews_connection_dup_password	(EEwsConnection *cnc);
 void		e_ews_connection_set_password	(EEwsConnection *cnc,
@@ -424,6 +446,7 @@ SoupSession *	e_ews_connection_ref_soup_session
 						(EEwsConnection *cnc);
 EEwsConnection *e_ews_connection_find		(const gchar *uri,
 						 const gchar *username);
+GSList *	e_ews_connection_list_existing	(void); /* EEwsConnection * */
 void		e_ews_connection_queue_request	(EEwsConnection *cnc,
 						 ESoapMessage *msg,
 						 EEwsResponseCallback cb,
@@ -431,12 +454,14 @@ void		e_ews_connection_queue_request	(EEwsConnection *cnc,
 						 GCancellable *cancellable,
 						 GSimpleAsyncResult *simple);
 
-gboolean	e_ews_autodiscover_ws_url_sync	(CamelEwsSettings *settings,
+gboolean	e_ews_autodiscover_ws_url_sync	(ESource *source,
+						 CamelEwsSettings *settings,
 						 const gchar *email_address,
 						 const gchar *password,
 						 GCancellable *cancellable,
 						 GError **error);
-void		e_ews_autodiscover_ws_url	(CamelEwsSettings *settings,
+void		e_ews_autodiscover_ws_url	(ESource *source,
+						 CamelEwsSettings *settings,
 						 const gchar *email_address,
 						 const gchar *password,
 						 GCancellable *cancellable,
@@ -1302,6 +1327,26 @@ gboolean	e_ews_connection_get_server_time_zones_sync
 						 GSList **tzds, /* EEwsCalendarTimeZoneDefinition */
 						 GCancellable *cancellable,
 						 GError **error);
+void		e_ews_connection_get_user_photo	(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *email,
+						 EEwsSizeRequested size_requested,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_user_photo_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 gchar **out_picture_data, /* base64-encoded */
+						 GError **error);
+gboolean	e_ews_connection_get_user_photo_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *email,
+						 EEwsSizeRequested size_requested,
+						 gchar **out_picture_data, /* base64-encoded */
+						 GCancellable *cancellable,
+						 GError **error);
 
 G_END_DECLS
 
